"""Goals serializers."""

from drf_writable_nested.serializers import WritableNestedModelSerializer

from dfhir.base.serializers import (
    AnnotationSerializer,
    BaseReferenceModelSerializer,
    BaseWritableNestedModelSerializer,
    CodeableConceptSerializer,
    DurationSerializer,
    IdentifierSerializer,
    QuantitySerializer,
    RangeSerializer,
    RatioSerializer,
)

from .models import (
    Goal,
    GoalAcceptance,
    GoalAcceptanceParticipantReference,
    GoalAddressesReference,
    GoalSourceReference,
    GoalSubjectReference,
    GoalTarget,
)


class GoalSubjectReferenceSerializer(BaseReferenceModelSerializer):
    """Goal Subject Reference serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """Meta class."""

        model = GoalSubjectReference
        exclude = ["created_at", "updated_at"]


class GoalAcceptanceParticipantReferenceSerializer(BaseReferenceModelSerializer):
    """Goal Acceptance Participant Reference serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """Meta class."""

        model = GoalAcceptanceParticipantReference
        exclude = ["created_at", "updated_at"]


class GoalAcceptanceSerializer(WritableNestedModelSerializer):
    """Goal Acceptance serializer."""

    participant = GoalAcceptanceParticipantReferenceSerializer(required=False)
    priority = CodeableConceptSerializer(required=False)

    class Meta:
        """Meta class."""

        model = GoalAcceptance
        exclude = ["created_at", "updated_at"]


class GoalTargetSerializer(WritableNestedModelSerializer):
    """Goal Target serializer."""

    measure = CodeableConceptSerializer(required=False)
    detail_quantity = QuantitySerializer(required=False)
    detail_range = RangeSerializer(required=False)
    detail_codeable_concept = CodeableConceptSerializer(required=False)
    detail_ratio = RatioSerializer(required=False)
    due_duration = DurationSerializer(required=False)

    class Meta:
        """Meta class."""

        model = GoalTarget
        exclude = ["created_at", "updated_at"]


class GoalSourceReferenceSerializer(BaseReferenceModelSerializer):
    """Goal Source Reference serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """Meta class."""

        model = GoalSourceReference
        exclude = ["created_at", "updated_at"]


class GoalAddressesReferenceSerializer(BaseReferenceModelSerializer):
    """Goal Addresses Reference serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """Meta class."""

        model = GoalAddressesReference
        exclude = ["created_at", "updated_at"]


class GoalSerializer(BaseWritableNestedModelSerializer):
    """Goal serializer."""

    identifier = IdentifierSerializer(many=True, required=False)
    achievement_status = CodeableConceptSerializer(required=False)
    category = CodeableConceptSerializer(many=True, required=False)
    priority = CodeableConceptSerializer(required=False)
    description = CodeableConceptSerializer(required=False)
    subject = GoalSubjectReferenceSerializer(required=False)
    start_codeable_concept = CodeableConceptSerializer(required=False)
    acceptance = GoalAcceptanceSerializer(required=False, many=True)
    target = GoalTargetSerializer(many=True, required=False)
    status_reason = CodeableConceptSerializer(required=False, many=True)
    source = GoalSourceReferenceSerializer(required=False)
    addresses = GoalAddressesReferenceSerializer(many=True, required=False)
    note = AnnotationSerializer(many=True, required=False)

    class Meta:
        """Meta class."""

        model = Goal
        exclude = ["created_at", "updated_at"]



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0004_initial'),
        ('careplans', '0003_initial'),
        ('conditions', '0002_initial'),
        ('medicationadministrations', '0002_initial'),
        ('patients', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='careplan',
            name='subject',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='care_plan_subject', to='patients.patientgroupreference'),
        ),
        migrations.AddField(
            model_name='careplan',
            name='supporting_info',
            field=models.ManyToManyField(blank=True, related_name='care_plan_supporting_info', to='base.reference'),
        ),
        migrations.AddField(
            model_name='careplanactivity',
            name='performer_activity',
            field=models.ManyToManyField(blank=True, related_name='care_plan_activity_performer_activity', to='base.codeablereference'),
        ),
        migrations.AddField(
            model_name='careplanactivity',
            name='progress',
            field=models.ManyToManyField(blank=True, related_name='care_plan_activity_progress', to='base.annotation'),
        ),
        migrations.AddField(
            model_name='careplan',
            name='activity',
            field=models.ManyToManyField(blank=True, related_name='care_plan_activity', to='careplans.careplanactivity'),
        ),
        migrations.AddField(
            model_name='careplanaddressescodeablereference',
            name='concept',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='care_plan_addresses_concept', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='careplan',
            name='addresses',
            field=models.ManyToManyField(blank=True, related_name='care_plan_addresses', to='careplans.careplanaddressescodeablereference'),
        ),
        migrations.AddField(
            model_name='careplanaddressreference',
            name='condition',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='conditions.condition'),
        ),
        migrations.AddField(
            model_name='careplanaddressreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='care_plan_address_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='careplanaddressreference',
            name='medication_administration',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='medicationadministrations.medicationadministration'),
        ),
        migrations.AddField(
            model_name='careplanaddressescodeablereference',
            name='reference',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='care_plan_addresses_codeable_reference', to='careplans.careplanaddressreference'),
        ),
        migrations.AddField(
            model_name='careplanbasedonreference',
            name='care_plan',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='care_plan_based_on_care_plan', to='careplans.careplan'),
        ),
        migrations.AddField(
            model_name='careplanbasedonreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='care_plan_based_on_identifier', to='base.identifier'),
        ),
    ]



"""image selection views."""

from django.http import Http404
from drf_spectacular.utils import extend_schema
from rest_framework import status
from rest_framework.permissions import AllowAny
from rest_framework.response import Response
from rest_framework.views import APIView

from dfhir.imagingselections.models import ImagingSelection
from dfhir.imagingselections.serializers import ImagingSelectionSerializer


class ImagingSelectionListView(APIView):
    """ImagingSelection list view."""

    permission_classes = [AllowAny]

    @extend_schema(responses={200: ImagingSelectionSerializer(many=True)})
    def get(self, request):
        """Get a list of imaging selections."""
        imaging_selections = ImagingSelection.objects.all()
        serializer = ImagingSelectionSerializer(imaging_selections, many=True)
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(
        request=ImagingSelectionSerializer, responses={201: ImagingSelectionSerializer}
    )
    def post(self, request):
        """Create an imaging selection."""
        serializer = ImagingSelectionSerializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data, status=status.HTTP_201_CREATED)


class ImagingSelectionDetailView(APIView):
    """imaging selection detail view."""

    def get_object(self, pk):
        """Get imaging selection object."""
        try:
            return ImagingSelection.objects.get(pk=pk)
        except ImagingSelection.DoesNotExist as err:
            raise Http404 from err

    @extend_schema(responses={200: ImagingSelectionSerializer})
    def get(self, request, pk=None):
        """Get an imaging selection."""
        imaging_selection = self.get_object(pk)
        serializer = ImagingSelectionSerializer(imaging_selection)
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(
        request=ImagingSelectionSerializer, responses={200: ImagingSelectionSerializer}
    )
    def patch(self, request, pk=None):
        """Update an imaging selection."""
        imaging_selection = self.get_object(pk)
        serializer = ImagingSelectionSerializer(
            imaging_selection, data=request.data, partial=True
        )
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(responses={204: None})
    def delete(self, request, pk=None):
        """Delete an imaging selection."""
        imaging_selection = self.get_object(pk)
        imaging_selection.delete()
        return Response(status=status.HTTP_204_NO_CONTENT)



"""Healthcare Services models."""

from django.db import models

from dfhir.base.models import (
    Attachment,
    Availability,
    BaseReference,
    CodeableConcept,
    ExtendedContactDetail,
    Identifier,
    OrganizationReference,
    Period,
    Quantity,
    Range,
    Reference,
    TimeStampedModel,
)


class HealthCareServiceReference(BaseReference):
    """Healthcare Service Reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="healthcareservice_reference_identifier",
    )
    healthcare_service = models.ForeignKey(
        "HealthcareService",
        on_delete=models.DO_NOTHING,
        related_name="healthcareservice_reference",
        null=True,
    )


class HealthCareServiceCodeableReference(TimeStampedModel):
    """CodeableReference model."""

    reference = models.ForeignKey(
        HealthCareServiceReference,
        on_delete=models.DO_NOTHING,
        related_name="codeable_reference",
        null=True,
    )
    concept = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        related_name="healthcareservice_codeableconcept",
        null=True,
    )


class ClinicalSpecialty(TimeStampedModel):
    """Clinical Specialty model."""

    code = models.CharField(max_length=255, null=True)
    display = models.CharField(max_length=255)


class ServiceCategory(TimeStampedModel):
    """Service Category model."""

    code = models.CharField(max_length=255, null=True)
    display = models.CharField(max_length=255, null=True)


class HealthCareServiceEligibilityValue(TimeStampedModel):
    """Healthcare Service Eligibility Value model."""

    value_codeable_concept = models.ForeignKey(
        CodeableConcept, on_delete=models.DO_NOTHING, null=True
    )
    value_boolean = models.BooleanField(null=True)
    value_quantity = models.ForeignKey(Quantity, on_delete=models.DO_NOTHING, null=True)
    value_range = models.ForeignKey(Range, on_delete=models.DO_NOTHING, null=True)
    value_reference = models.ForeignKey(
        Reference, on_delete=models.DO_NOTHING, null=True
    )


class HealthcareServiceEligibility(TimeStampedModel):
    """Healthcare Service Eligibility model."""

    code = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="healthcareserviceeligibility_code",
    )
    value = models.ForeignKey(
        HealthCareServiceEligibilityValue, on_delete=models.DO_NOTHING, null=True
    )
    comment = models.TextField(null=True)
    period = models.ForeignKey(Period, on_delete=models.DO_NOTHING, null=True)


class HealthcareService(TimeStampedModel):
    """Healthcare Service model."""

    identifier = models.ManyToManyField(
        Identifier, related_name="healthcareservice_identifier", blank=True
    )
    active = models.BooleanField(default=True)
    offered_in = models.ManyToManyField(
        HealthCareServiceReference,
        related_name="healthcareservice_offered_in",
        blank=True,
    )
    name = models.CharField(max_length=255)
    location = models.ManyToManyField(
        "locations.LocationReference",
        related_name="healthcare_service_location",
        blank=True,
    )
    provided_by = models.ForeignKey(
        OrganizationReference, on_delete=models.DO_NOTHING, null=True
    )
    comment = models.TextField(null=True)
    extra_details = models.TextField(null=True)
    photo = models.ForeignKey(Attachment, on_delete=models.CASCADE, null=True)
    coverage_area = models.ManyToManyField(
        "locations.LocationReference",
        related_name="healthcareservice_coverage_area",
        blank=True,
    )
    referral_required = models.BooleanField(default=False)
    referral_method = models.ManyToManyField(
        CodeableConcept, related_name="healthcareservice_referral_method", blank=True
    )
    appointment_required = models.BooleanField(default=False)
    type = models.ManyToManyField(
        CodeableConcept, blank=True, related_name="healthcareservice_type"
    )
    specialty = models.ManyToManyField(
        CodeableConcept, blank=True, related_name="healthcareservice_specialty"
    )
    category = models.ManyToManyField(
        CodeableConcept, blank=True, related_name="healthcareservice_category"
    )
    service_provision_code = models.ManyToManyField(
        CodeableConcept,
        blank=True,
        related_name="healthcareservice_service_provision_code",
    )
    program = models.ManyToManyField(
        CodeableConcept, related_name="healthcareservice_program", blank=True
    )
    characteristic = models.ManyToManyField(
        CodeableConcept, related_name="healthcareservice_characteristics", blank=True
    )
    communication = models.ManyToManyField(
        CodeableConcept, related_name="healthcareservice_communication", blank=True
    )
    contact = models.ManyToManyField(
        ExtendedContactDetail, related_name="healthcareservice_contact", blank=True
    )
    availability = models.ForeignKey(
        Availability,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="healthcareservice_availability",
    )
    endpoint = models.ManyToManyField(
        "endpoints.Endpoint", related_name="healthcareservice_endpoint", blank=True
    )
    eligibility = models.ManyToManyField(
        HealthcareServiceEligibility,
        related_name="healthcareservice_eligibility",
        blank=True,
    )



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0002_initial'),
        ('devicedefinitions', '0001_initial'),
        ('devicemetrics', '0001_initial'),
        ('endpoints', '0001_initial'),
        ('locations', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='DeviceUdiCarrier',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('device_identifier', models.CharField(max_length=255, null=True)),
                ('device_identifier_system', models.CharField(max_length=255, null=True)),
                ('carrier_hrft', models.CharField(max_length=255, null=True)),
                ('carrier_aidc', models.CharField(max_length=255, null=True)),
                ('issuer', models.CharField(max_length=255, null=True)),
                ('jurisdiction', models.CharField(max_length=255, null=True)),
                ('carrier', models.CharField(max_length=255, null=True)),
                ('entry_type', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Device',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('status', models.CharField(choices=[('active', 'Active'), ('inactive', 'Inactive'), ('entered-in-error', 'Entered in Error')], max_length=255, null=True)),
                ('manufacturer', models.CharField(max_length=255, null=True)),
                ('manufacture_date', models.DateTimeField(null=True)),
                ('expiration_date', models.DateTimeField(null=True)),
                ('lot_number', models.CharField(max_length=255, null=True)),
                ('serial_number', models.CharField(max_length=255, null=True)),
                ('model_number', models.CharField(max_length=255, null=True)),
                ('part_number', models.CharField(max_length=255, null=True)),
                ('cycle', models.IntegerField(null=True)),
                ('url', models.URLField(null=True)),
                ('availability_status', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='devices_availability_status', to='base.codeableconcept')),
                ('biological_source_event', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='devices_biological_source_event', to='base.identifier')),
                ('category', models.ManyToManyField(blank=True, related_name='devices_category', to='base.codeableconcept')),
                ('contact', models.ManyToManyField(blank=True, related_name='devices_contact', to='base.contactpoint')),
                ('definition', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to='devicedefinitions.devicedefinitionreference')),
                ('duration', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='devices_duration', to='base.quantity')),
                ('endpoint', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='devices_endpoint', to='endpoints.endpointreference')),
                ('identifier', models.ManyToManyField(blank=True, related_name='devices_identifier', to='base.identifier')),
                ('location', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='devices_location', to='locations.locationreference')),
                ('mode', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='devices_mode', to='base.codeableconcept')),
                ('note', models.ManyToManyField(blank=True, related_name='devices_note', to='base.annotation')),
                ('owner', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='devices_owner', to='base.organizationreference')),
                ('safety', models.ManyToManyField(blank=True, related_name='devices_safety', to='base.codeableconcept')),
                ('type', models.ManyToManyField(blank=True, related_name='devices_type', to='base.codeableconcept')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='DeviceCodeableReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('concept', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='device_codeable_reference_concept', to='base.codeableconcept')),
                ('device', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='device_reference_device', to='devices.device')),
                ('identifier', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='device_reference_identifier', to='base.identifier')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='device',
            name='gateway',
            field=models.ManyToManyField(blank=True, related_name='devices_gateway', to='devices.devicecodeablereference'),
        ),
        migrations.CreateModel(
            name='DeviceConformsTo',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('version', models.CharField(max_length=255, null=True)),
                ('category', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='device_conforms_to_category', to='base.codeableconcept')),
                ('specification', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='device_conforms_to_specification', to='base.codeableconcept')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='device',
            name='conforms_to',
            field=models.ManyToManyField(blank=True, related_name='devices_conforms_to', to='devices.deviceconformsto'),
        ),
        migrations.CreateModel(
            name='DeviceDeviceDefinitionReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
                ('device', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='device_device_definition_reference_device', to='devices.device')),
                ('device_definition', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='device_device_definition_reference_device_definition', to='devicedefinitions.devicedefinition')),
                ('identifier', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='device_device_definition_reference_identifier', to='base.identifier')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='DeviceDeviceDefinitionCodeableReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('concept', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='device_device_definition_codeable_reference_concept', to='base.codeableconcept')),
                ('reference', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='device_device_definition_codeable_reference_reference', to='devices.devicedevicedefinitionreference')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='DeviceDeviceMetricReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
                ('device', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='device_device_metric_reference_device', to='devices.device')),
                ('device_metric', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='device_device_metric_reference_device_metric', to='devicemetrics.devicemetric')),
                ('identifier', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='device_device_metric_reference_identifier', to='base.identifier')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='DeviceName',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('value', models.CharField(max_length=255, null=True)),
                ('display', models.BooleanField(default=False, null=True)),
                ('type', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='device_name_type', to='base.codeableconcept')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='device',
            name='name',
            field=models.ManyToManyField(blank=True, related_name='devices_name', to='devices.devicename'),
        ),
        migrations.CreateModel(
            name='DeviceProperty',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('value_string', models.CharField(max_length=255, null=True)),
                ('value_boolean', models.BooleanField(null=True)),
                ('value_integer', models.IntegerField(null=True)),
                ('type', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='device_property_type', to='base.codeableconcept')),
                ('value_attachment', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='device_property_value_attachment', to='base.attachment')),
                ('value_codeable_concept', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='device_property_value_code', to='base.codeableconcept')),
                ('value_quantity', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='device_property_value_quantity', to='base.quantity')),
                ('value_range', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='device_property_value_range', to='base.range')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='device',
            name='property',
            field=models.ManyToManyField(blank=True, related_name='devices_property', to='devices.deviceproperty'),
        ),
        migrations.CreateModel(
            name='DeviceReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
                ('device', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to='devices.device')),
                ('identifier', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to='base.identifier')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='devicecodeablereference',
            name='reference',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='device_codeable_reference_reference', to='devices.devicereference'),
        ),
        migrations.AddField(
            model_name='device',
            name='parent',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='devices_parent', to='devices.devicereference'),
        ),
        migrations.AddField(
            model_name='device',
            name='udi_carrier',
            field=models.ManyToManyField(blank=True, to='devices.deviceudicarrier'),
        ),
        migrations.CreateModel(
            name='DeviceVersion',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('value', models.CharField(max_length=255, null=True)),
                ('install_date', models.DateTimeField(null=True)),
                ('component', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='device_version_component', to='base.identifier')),
                ('type', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='device_version_type', to='base.codeableconcept')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='device',
            name='device_version',
            field=models.ManyToManyField(blank=True, related_name='devices_device_version', to='devices.deviceversion'),
        ),
    ]



"""molecular sequences serializers."""

from drf_writable_nested.serializers import WritableNestedModelSerializer
from rest_framework import serializers

from dfhir.base.serializers import (
    AttachmentSerializer,
    BaseReferenceModelSerializer,
    CodeableConceptSerializer,
    IdentifierSerializer,
)
from dfhir.molecularsequences.models import (
    MolecularSequence,
    MolecularSequenceConcatenated,
    MolecularSequenceConcatenatedSequenceElement,
    MolecularSequenceExtracted,
    MolecularSequenceLiteral,
    MolecularSequenceReference,
    MolecularSequenceRelative,
    MolecularSequenceRelativeEdit,
    MolecularSequenceRepeated,
)


class MolecularSequenceReferenceSerializer(BaseReferenceModelSerializer):
    """molecular reference sreializer."""

    identifier = IdentifierSerializer(many=False, required=False)

    class Meta:
        """Meta options."""

        model = MolecularSequenceReference
        exclude = ["created_at", "updated_at"]


class MolecularSequenceLiteralSerializer(serializers.ModelSerializer):
    """molecular sequence literal serializer."""

    class Meta:
        """Meta options."""

        model = MolecularSequenceLiteral
        exclude = ["created_at", "updated_at"]


class MolecularSequenceRelativeEditSerializer(WritableNestedModelSerializer):
    """molecular sequence relative edit serializer."""

    coordinate_system = CodeableConceptSerializer(many=False, required=False)
    replacement_sequence = MolecularSequenceReferenceSerializer(
        many=False, required=False
    )
    replaced_sequence = MolecularSequenceReferenceSerializer(many=False, required=False)

    class Meta:
        """Meta options."""

        model = MolecularSequenceRelativeEdit
        exclude = ["created_at", "updated_at"]


class MolecularSequenceRelativeSerializer(WritableNestedModelSerializer):
    """molecular sequence relative serializer."""

    starting_sequence = MolecularSequenceReferenceSerializer(many=False, required=False)
    relative_edit = MolecularSequenceRelativeEditSerializer(many=True, required=False)

    class Meta:
        """Meta options."""

        model = MolecularSequenceRelative
        exclude = ["created_at", "updated_at"]


class MolecularSequenceExtractedSerializer(WritableNestedModelSerializer):
    """molecular sequence extracted serializer."""

    starting_sequence = MolecularSequenceReferenceSerializer(many=False, required=False)
    coordinated_system = CodeableConceptSerializer(many=False, required=False)

    class Meta:
        """Metaclass."""

        model = MolecularSequenceExtracted
        exclude = ["created_at", "updated_at"]


class MolecularSequenceRepeatedSerializer(WritableNestedModelSerializer):
    """molecular sequence repeated serializer."""

    sequence_motif = CodeableConceptSerializer(many=False, required=False)

    class Meta:
        """Meta options."""

        model = MolecularSequenceRepeated
        exclude = ["created_at", "updated_at"]


class MolecularSequenceConcatenatedSequenceElementSerializer(
    WritableNestedModelSerializer
):
    """molecular sequence concatenated sequence element serializer."""

    sequence = MolecularSequenceReferenceSerializer(many=False, required=False)

    class Meta:
        """Meta options."""

        model = MolecularSequenceConcatenatedSequenceElement
        exclude = ["created_at", "updated_at"]


class MolecularSequenceConcatenatedSerializer(WritableNestedModelSerializer):
    """molecular sequence concatenated serializer."""

    sequence = MolecularSequenceReferenceSerializer(many=False, required=False)

    class Meta:
        """Meta options."""

        model = MolecularSequenceConcatenated
        exclude = ["created_at", "updated_at"]


class MolecularSequenceSerializer(WritableNestedModelSerializer):
    """molecular sequence serializer."""

    identifier = IdentifierSerializer(many=True, required=False)
    literal = MolecularSequenceLiteralSerializer(many=False, required=False)
    file = AttachmentSerializer(many=True, required=False)
    relative = MolecularSequenceRelativeSerializer(many=True, required=False)
    extracted = MolecularSequenceExtractedSerializer(many=True, required=False)
    repeated = MolecularSequenceRepeatedSerializer(many=True, required=False)
    concatenated = MolecularSequenceConcatenatedSerializer(many=True, required=False)

    class Meta:
        """Meta options."""

        model = MolecularSequence
        exclude = ["created_at", "updated_at"]



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0005_initial'),
        ('conditions', '0004_initial'),
        ('diagnosticreports', '0003_initial'),
        ('medicationstatements', '0003_initial'),
        ('observations', '0002_initial'),
        ('procedures', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='medicationstatementpartofreference',
            name='procedure',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='medication_statement_part_of_reference_procedure', to='procedures.procedure'),
        ),
        migrations.AddField(
            model_name='medicationstatement',
            name='part_of',
            field=models.ManyToManyField(blank=True, related_name='medication_statement_part_of', to='medicationstatements.medicationstatementpartofreference'),
        ),
        migrations.AddField(
            model_name='medicationstatementreasonreference',
            name='condition',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='medication_statement_reason_reference_condition', to='conditions.condition'),
        ),
        migrations.AddField(
            model_name='medicationstatementreasonreference',
            name='diagnostic_report',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='medication_statement_reason_reference_diagnostic_report', to='diagnosticreports.diagnosticreport'),
        ),
        migrations.AddField(
            model_name='medicationstatementreasonreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='medication_statement_reason_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='medicationstatementreasonreference',
            name='observation',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='medication_statement_reason_reference_observation', to='observations.observation'),
        ),
        migrations.AddField(
            model_name='medicationstatementreasonreference',
            name='procedure',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='medication_statement_reason_reference_procedure', to='procedures.procedure'),
        ),
        migrations.AddField(
            model_name='medicationstatement',
            name='reason',
            field=models.ManyToManyField(blank=True, related_name='medication_statement_reason', to='medicationstatements.medicationstatementreasonreference'),
        ),
    ]



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0005_initial'),
        ('careteams', '0003_initial'),
        ('practitionerroles', '0001_initial'),
        ('practitioners', '0001_initial'),
        ('relatedpersons', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='careteamparticipantonbehalfofreference',
            name='practitioner',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to='practitioners.practitioner'),
        ),
        migrations.AddField(
            model_name='careteamparticipantonbehalfofreference',
            name='practitioner_role',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to='practitionerroles.practitionerrole'),
        ),
        migrations.AddField(
            model_name='careteamparticipantonbehalfofreference',
            name='related_person',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to='relatedpersons.relatedperson'),
        ),
        migrations.AddField(
            model_name='careteamparticipant',
            name='on_behalf_of',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='care_team_participants_on_behalf_of', to='careteams.careteamparticipantonbehalfofreference'),
        ),
        migrations.AddField(
            model_name='careteamreference',
            name='care_team',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='care_team_reference_care_team', to='careteams.careteam'),
        ),
        migrations.AddField(
            model_name='careteamreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='care_team_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='careteamparticipant',
            name='member',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='care_team_participants_member', to='careteams.careteamparticipantmemberreference'),
        ),
    ]



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0003_initial'),
        ('encounters', '0002_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='NutritionOrderAdditive',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('product_name', models.CharField(blank=True, max_length=255)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='NutritionOrderBasedOnReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='NutritionOrderCodeableReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='NutritionOrderEnteralFormula',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('product_name', models.CharField(max_length=255, null=True)),
                ('administration_instruction', models.TextField(null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='NutritionOrderEnteralFormulaAdministration',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='NutritionOrderOralDiet',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('instruction', models.TextField(blank=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='NutritionOrderOralDietNutrient',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='NutritionOrderOralDietTexture',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='NutritionOrderPerformerReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='NutritionOrderReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='NutritionOrderSchedule',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('as_needed', models.BooleanField(default=False)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='NutritionOrderSupplement',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('productName', models.CharField(blank=True, max_length=255)),
                ('instruction', models.TextField(blank=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='NutritionOrder',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('instantiates_uri', models.URLField(null=True)),
                ('instantiates', models.URLField(null=True)),
                ('status', models.CharField(blank=True, choices=[('draft', 'Draft'), ('active', 'Active'), ('on-hold', 'On Hold'), ('revoked', 'Revoked'), ('completed', 'Completed'), ('entered-in-error', 'Entered in Error'), ('unknown', 'Unknown')], max_length=255)),
                ('intent', models.CharField(choices=[('proposal', 'Proposal'), ('plan', 'Plan'), ('order', 'Order'), ('directive-order', 'Directive Order'), ('original-order', 'Original Order'), ('reflex-order', 'Reflex Order'), ('filler-order', 'Filler Order'), ('instance-order', 'Instance Order'), ('option', 'Option')], max_length=255, null=True)),
                ('priority', models.CharField(choices=[('routine', 'Routine'), ('urgent', 'Urgent'), ('stat', 'Stat'), ('asap', 'ASAP')], max_length=255, null=True)),
                ('date_time', models.DateTimeField(null=True)),
                ('outside_food_allowed', models.BooleanField(default=False)),
                ('encounter', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='nutrition_order_encounter', to='encounters.encounterreference')),
                ('exclude_food_modifier', models.ManyToManyField(blank=True, related_name='nutrition_order_exclude_food_modifier', to='base.codeableconcept')),
                ('food_preference_modifier', models.ManyToManyField(blank=True, related_name='nutrition_order_food_preference_modifier', to='base.codeableconcept')),
                ('group_identifier', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='nutrition_order_group_identifier', to='base.identifier')),
                ('identifier', models.ManyToManyField(blank=True, related_name='nutrition_order_identifier', to='base.identifier')),
                ('note', models.ManyToManyField(blank=True, related_name='nutrition_order_note', to='base.annotation')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0003_initial'),
        ('devices', '0001_initial'),
        ('encounters', '0002_initial'),
        ('medications', '0001_initial'),
        ('provenances', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='AdditionalIllustration',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('code', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='DosageMethod',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('code', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='DosageRoute',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('code', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='DosageSite',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('code', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='MedicationRequestBasedOnReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='MedicationRequestCategory',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('display', models.CharField(max_length=255, null=True)),
                ('description', models.TextField(null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='medicationRequestInformationSourceReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='MedicationRequestInsuranceReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='MedicationRequestMedicationCode',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('code', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='MedicationRequestPerformerReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='MedicationRequestProvinanceReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='MedicationRequestReason',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('code', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='MedicationRequestReasonReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='MedicationRequestReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='MedicationRequestReferenceType',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('display', models.CharField(max_length=255, null=True)),
                ('description', models.TextField(null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='MedicationRequestRequesterReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='MedicationRequestSubstitution',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('allowed_boolean', models.BooleanField(default=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='MedicatonRequestReasonCodealbleReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ReferenceAsNeededFor',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('code', models.CharField(max_length=255)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='DispenseRequestInitialFill',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('duration', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='dispense_request_initial_fill_duration', to='base.quantity')),
                ('quantity', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='dispense_request_initial_fill_quantity', to='base.quantity')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='DispenseRequest',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('number_of_repeats_allowed', models.IntegerField(null=True)),
                ('dispense_instruction', models.ManyToManyField(blank=True, related_name='dispense_request_instruction', to='base.annotation')),
                ('dispense_interval', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='dispense_request_dispense_interval', to='base.quantity')),
                ('dispenser', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='dispense_request_dispenser', to='base.organizationreference')),
                ('dose_administration_aid', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='dispense_request_dose_administration_aid', to='base.codeableconcept')),
                ('expected_supply_duration', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='base.quantity')),
                ('quantity', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='dispense_request_quantity', to='base.quantity')),
                ('validity_period', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='dispense_request_validity_period', to='base.period')),
                ('initial_fill', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='dispense_request_initial_fill', to='medicationrequests.dispenserequestinitialfill')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='MedicationRequest',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('status', models.CharField(choices=[('active', 'Active'), ('on_hold', 'On hold'), ('ended', 'Ended'), ('stopped', 'Stopped'), ('completed', 'Completed'), ('cancelled', 'Cancelled'), ('entered_in_error', 'Entered in error'), ('draft', 'Draft'), ('unknown', 'Unknown')], max_length=255, null=True)),
                ('status_changed', models.DateTimeField(null=True)),
                ('intent', models.CharField(choices=[('plan', 'Plan'), ('order', 'Order'), ('original_order', 'Original order'), ('filler_order', 'Filler order'), ('instance_order', 'Instance order'), ('option', 'Option')], max_length=255, null=True)),
                ('priority', models.CharField(choices=[('routine', 'Routine'), ('urgent', 'Urgent'), ('asap', 'ASAP'), ('stat', 'STAT')], max_length=255, null=True)),
                ('do_not_perform', models.BooleanField(default=False)),
                ('authored_on', models.DateTimeField(null=True)),
                ('reported', models.BooleanField(default=True)),
                ('rendered_dosage_instruction', models.TextField(null=True)),
                ('category', models.ManyToManyField(blank=True, related_name='medication_request_category', to='base.codeableconcept')),
                ('course_of_therapy_type', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='medication_request_course_of_therapy_type', to='base.codeableconcept')),
                ('device', models.ManyToManyField(blank=True, related_name='medication_request_device', to='devices.devicecodeablereference')),
                ('dispense_request', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='medication_request_dispense_request', to='medicationrequests.dispenserequest')),
                ('effective_dose_period', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='medication_request_effective_dose_period', to='base.period')),
                ('encounter', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='medication_request_encounter', to='encounters.encounterreference')),
                ('event_history', models.ManyToManyField(blank=True, related_name='medication_request_event_history', to='provenances.provenancereference')),
                ('group_identifier', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='medication_request_group_identifier', to='base.identifier')),
                ('identifier', models.ManyToManyField(blank=True, related_name='medication_request_identifier', to='base.identifier')),
                ('medication', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='medication_request_mepatientdication', to='medications.medicationcodeablereference')),
                ('note', models.ManyToManyField(blank=True, related_name='medication_request_note', to='base.annotation')),
                ('performer_type', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='medication_request_performer_type', to='base.codeableconcept')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



"""CoverageEligibilityRequests views."""

from django.http import Http404
from drf_spectacular.utils import extend_schema
from rest_framework import status
from rest_framework.permissions import AllowAny
from rest_framework.response import Response
from rest_framework.views import APIView

from .models import CoverageEligibilityRequest
from .serializers import CoverageEligibilityRequestSerializer


class CoverageEligibilityRequestListView(APIView):
    """CoverageEligibilityRequest list create view."""

    permission_classes = [AllowAny]

    @extend_schema(
        request=CoverageEligibilityRequestSerializer,
        responses={201: CoverageEligibilityRequestSerializer},
    )
    def post(self, request):
        """Create a coverage eligibility request."""
        serializer = CoverageEligibilityRequestSerializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data, status=status.HTTP_201_CREATED)

    @extend_schema(responses={200: CoverageEligibilityRequestSerializer(many=True)})
    def get(self, request):
        """Get all coverage eligibility requests."""
        coverage_eligibility_requests = CoverageEligibilityRequest.objects.all()
        serializer = CoverageEligibilityRequestSerializer(
            coverage_eligibility_requests, many=True
        )
        return Response(serializer.data, status=status.HTTP_200_OK)


class CoverageEligibilityRequestDetailView(APIView):
    """CoverageEligibilityRequest detail view."""

    permission_classes = [AllowAny]

    def get_object(self, pk):
        """Get coverage eligibility request object."""
        try:
            return CoverageEligibilityRequest.objects.get(id=pk)
        except CoverageEligibilityRequest.DoesNotExist as err:
            raise Http404 from err

    @extend_schema(responses={200: CoverageEligibilityRequestSerializer})
    def get(self, request, pk=None):
        """Get coverage eligibility request detail."""
        coverage_eligibility_request = self.get_object(pk)
        serializer = CoverageEligibilityRequestSerializer(coverage_eligibility_request)
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(
        request=CoverageEligibilityRequestSerializer,
        responses={200: CoverageEligibilityRequestSerializer},
    )
    def patch(self, request, pk=None):
        """Update coverage eligibility request."""
        coverage_eligibility_request = self.get_object(pk)
        serializer = CoverageEligibilityRequestSerializer(
            coverage_eligibility_request, data=request.data, partial=True
        )
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(responses={204: None})
    def delete(self, request, pk=None):
        """Delete coverage eligibility request."""
        coverage_eligibility_request = self.get_object(pk)
        coverage_eligibility_request.delete()
        return Response(status=status.HTTP_204_NO_CONTENT)



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('detectedissues', '0004_initial'),
        ('procedures', '0001_initial'),
        ('substances', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='detectedissuesubjectreference',
            name='procedure',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='detected_issue_subject_reference_procedure', to='procedures.procedure'),
        ),
        migrations.AddField(
            model_name='detectedissuesubjectreference',
            name='substance',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='detected_issue_subject_reference_substance', to='substances.substance'),
        ),
        migrations.AddField(
            model_name='detectedissue',
            name='subject',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='detected_issue_patient', to='detectedissues.detectedissuesubjectreference'),
        ),
    ]



"""care plan serializers."""

from drf_writable_nested.serializers import WritableNestedModelSerializer

from dfhir.base.serializers import (
    AnnotationSerializer,
    BaseReferenceModelSerializer,
    BaseWritableNestedModelSerializer,
    CodeableConceptSerializer,
    CodeableReferenceSerializer,
    IdentifierSerializer,
    PeriodSerializer,
    ReferenceSerializer,
)
from dfhir.careteams.serializers import CareTeamReferenceSerializer
from dfhir.encounters.serializers import EncounterReferenceSerializer
from dfhir.patients.serializers import PatientGroupReferenceSerializer

from .models import (
    CarePlan,
    CarePlanActivity,
    CarePlanAddressesCodeableReference,
    CarePlanAddressReference,
    CarePlanBasedOnReference,
    CarePlanContributorReference,
    CarePlanCustodianReference,
    CarePlanDeviceRequestReference,
    CarePlanPlannedActivityReference,
    CarePlanReference,
)


class CarePlanBasedOnReferenceSerializer(BaseReferenceModelSerializer):
    """Care Plan Based On Reference Serializer."""

    identifier = IdentifierSerializer(required=False, many=False)

    class Meta:
        """Meta."""

        model = CarePlanBasedOnReference
        exclude = ["created_at", "updated_at"]


class CarePlanContributorReferenceSerializer(BaseReferenceModelSerializer):
    """Care Plan Contributor Reference Serializer."""

    identifier = IdentifierSerializer(required=False, many=False)

    class Meta:
        """Meta."""

        model = CarePlanContributorReference
        exclude = ["created_at", "updated_at"]


class CarePlanCustodianReferenceSerializer(BaseReferenceModelSerializer):
    """Care Plan Custodian Reference Serializer."""

    identifier = IdentifierSerializer(required=False, many=False)

    class Meta:
        """Meta."""

        model = CarePlanCustodianReference
        exclude = ["created_at", "updated_at"]


class CarePlanAddressReferenceSerializer(BaseReferenceModelSerializer):
    """Care Plan Address Reference Serializer."""

    identifier = IdentifierSerializer(required=False, many=False)

    class Meta:
        """Meta."""

        model = CarePlanAddressReference
        exclude = ["created_at", "updated_at"]


class CarePlanPlannedActivityReferenceSerializer(BaseReferenceModelSerializer):
    """Care Plan Planned Activity Reference Serializer."""

    identifier = IdentifierSerializer(required=False, many=False)

    class Meta:
        """Meta."""

        model = CarePlanPlannedActivityReference
        exclude = ["created_at", "updated_at"]


class CarePlanActivitySerializer(WritableNestedModelSerializer):
    """Care Plan Activity Serializer."""

    performer_activity = CodeableReferenceSerializer(required=False, many=True)
    progress = AnnotationSerializer(required=False, many=True)
    planned_activity_reference = CarePlanPlannedActivityReferenceSerializer(
        required=False, many=False
    )

    class Meta:
        """Meta."""

        model = CarePlanActivity
        exclude = ["created_at", "updated_at"]


class CarePlanReferenceSerializer(BaseReferenceModelSerializer):
    """Care Plan Reference Serializer."""

    identifier = IdentifierSerializer(required=False, many=False)

    class Meta:
        """Meta."""

        model = CarePlanReference
        exclude = ["created_at", "updated_at"]


class CarePlanAddressesCodeableReferenceSerializer(WritableNestedModelSerializer):
    """Care Plan Addresses Codeable Reference Serializer."""

    reference = CarePlanReferenceSerializer(required=False, many=False)
    concept = CodeableConceptSerializer(required=False, many=False)

    class Meta:
        """Meta."""

        model = CarePlanAddressesCodeableReference
        exclude = ["created_at", "updated_at"]


class CarePlanSerializer(BaseWritableNestedModelSerializer):
    """care plan serializer."""

    identifier = IdentifierSerializer(many=True, required=False)
    based_on = CarePlanBasedOnReferenceSerializer(many=True, required=False)
    replaces = CarePlanReferenceSerializer(many=True, required=False)
    part_of = CarePlanReferenceSerializer(many=True, required=False)
    category = CodeableConceptSerializer(many=True, required=False)
    subject = PatientGroupReferenceSerializer(required=False, many=False)
    encounter = EncounterReferenceSerializer(required=False, many=False)
    period = PeriodSerializer(required=False, many=False)
    custodian = CarePlanCustodianReferenceSerializer(many=False, required=False)
    contributor = CarePlanContributorReferenceSerializer(many=True, required=False)
    care_team = CareTeamReferenceSerializer(many=True, required=False)
    addresses = CarePlanAddressesCodeableReferenceSerializer(many=True, required=False)
    supporting_info = ReferenceSerializer(many=True, required=False)
    activity = CarePlanActivitySerializer(many=True, required=False)
    goal = ReferenceSerializer(many=True, required=False)

    class Meta:
        """meta options."""

        model = CarePlan
        exclude = ["created_at", "updated_at"]


class CarePlanDeviceRequestReferenceSerializer(BaseReferenceModelSerializer):
    """Care Plan Device Request Reference Serializer."""

    identifier = IdentifierSerializer(required=False, many=False)

    class Meta:
        """Meta."""

        model = CarePlanDeviceRequestReference
        exclude = ["created_at", "updated_at"]



# Generated by Django 5.2 on 2025-04-23 12:46

import django.contrib.postgres.fields
import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0003_initial'),
        ('groups', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='SpecimenDefinitionTypeTested',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('is_derived', models.BooleanField(null=True)),
                ('preference', models.CharField(choices=[('preferred', 'preferred'), ('alternate', 'alternate')], default='preferred', max_length=255)),
                ('requirements', models.TextField(null=True)),
                ('single_use', models.BooleanField(null=True)),
                ('retention_criterion', models.ManyToManyField(blank=True, related_name='specimen_definition_type_tested_retention_criterion', to='base.codeableconcept')),
                ('retention_time', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='specimen_definition_type_tested_retention_time', to='base.quantity')),
                ('testing_destination', models.ManyToManyField(blank=True, related_name='specimen_definition_type_tested_testing_destination', to='base.codeableconcept')),
                ('type', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='specimen_definition_type_tested_type', to='base.codeableconcept')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='SpecimenDefinition',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('uri', models.URLField(null=True)),
                ('version', models.CharField(max_length=255, null=True)),
                ('version_algorithm_string', models.CharField(max_length=255, null=True)),
                ('name', models.CharField(max_length=255, null=True)),
                ('title', models.CharField(max_length=255, null=True)),
                ('derived_from_uri', django.contrib.postgres.fields.ArrayField(base_field=models.URLField(), null=True, size=None)),
                ('status', models.CharField(choices=[('draft', 'draft'), ('active', 'active'), ('retired', 'retired'), ('unknown', 'unknown')], default='active', max_length=255)),
                ('experimental', models.BooleanField(null=True)),
                ('date', models.DateField(null=True)),
                ('publisher', models.CharField(max_length=255, null=True)),
                ('description', models.TextField(null=True)),
                ('purpose', models.TextField(null=True)),
                ('copyright', models.TextField(null=True)),
                ('copyright_label', models.CharField(max_length=255, null=True)),
                ('approval_date', models.DateField(null=True)),
                ('last_review_date', models.DateField(null=True)),
                ('time_aspect', models.CharField(max_length=255, null=True)),
                ('collection', models.ManyToManyField(blank=True, related_name='specimen_definition_collection', to='base.codeableconcept')),
                ('contact', models.ManyToManyField(blank=True, related_name='specimen_definition_contact', to='base.contactdetail')),
                ('effective_period', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='specimen_definition_effective_period', to='base.period')),
                ('identifier', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='specimen_definitions_identifier', to='base.identifier')),
                ('jurisdiction', models.ManyToManyField(blank=True, related_name='specimen_definition_jurisdiction', to='base.codeableconcept')),
                ('patient_preparation', models.ManyToManyField(blank=True, related_name='specimen_definition_patient_preparation', to='base.codeableconcept')),
                ('subject_codeable_concept', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='specimen_definition_subject_codeable_concept', to='base.codeableconcept')),
                ('subject_reference', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='specimen_definition_subject_reference', to='groups.groupreference')),
                ('type_collected', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='specimen_definition_type_collected', to='base.codeableconcept')),
                ('use_context', models.ManyToManyField(blank=True, related_name='specimen_definition_use_context', to='base.usagecontext')),
                ('version_algorithm_coding', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='specimen_definition_version_algorithm_coding', to='base.coding')),
                ('type_tested', models.ManyToManyField(blank=True, related_name='specimen_definition_type_tested', to='specimendefinitions.specimendefinitiontypetested')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='SpecimenDefinitionTypeTestedContainer',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('description', models.TextField(null=True)),
                ('minimum_volume_string', models.CharField(max_length=255, null=True)),
                ('cap', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='specimen_definition_type_tested_container_cap', to='base.codeableconcept')),
                ('capacity', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='specimen_definition_type_tested_container_capacity', to='base.quantity')),
                ('material', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='specimen_definition_type_tested_container_material', to='base.codeableconcept')),
                ('minimum_volume_quantity', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='specimen_definition_type_tested_container_minimum_volume_quantity', to='base.quantity')),
                ('type', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='specimen_definition_type_tested_container_type', to='base.codeableconcept')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='specimendefinitiontypetested',
            name='container',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='specimen_definition_type_tested_container', to='specimendefinitions.specimendefinitiontypetestedcontainer'),
        ),
        migrations.CreateModel(
            name='SpecimenDefinitionTypeTestedContainerAdditive',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('preparation', models.TextField(null=True)),
                ('additive_codeable_concept', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='specimen_definition_type_tested_container_additive_codeable_concept', to='base.codeableconcept')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='specimendefinitiontypetestedcontainer',
            name='additive',
            field=models.ManyToManyField(blank=True, related_name='specimen_definition_type_tested_container_additive', to='specimendefinitions.specimendefinitiontypetestedcontaineradditive'),
        ),
        migrations.CreateModel(
            name='SpecimenDefinitionTypeTestedHandling',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('instruction', models.TextField(null=True)),
                ('max_duration', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='specimen_definition_type_tested_handling_max_duration', to='base.quantity')),
                ('temperature_qualifier', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='specimen_definition_type_tested_handling_temperature_qualifier', to='base.codeableconcept')),
                ('temperature_range', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='specimen_definition_type_tested_handling_temperature_range', to='base.range')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='specimendefinitiontypetested',
            name='handling',
            field=models.ManyToManyField(blank=True, related_name='specimen_definition_type_tested_handling', to='specimendefinitions.specimendefinitiontypetestedhandling'),
        ),
    ]



"""Explanation of Benefits models."""

from django.contrib.postgres.fields import ArrayField
from django.db import models

from dfhir.base.models import (
    Address,
    Attachment,
    BaseReference,
    CodeableConcept,
    Coding,
    Identifier,
    Money,
    OrganizationReference,
    Period,
    Quantity,
    Reference,
    SimpleQuantity,
    TimeStampedModel,
)

from . import choices


class ExplanationOfBenefitEntererReference(BaseReference):
    """Explanation of Benefit enterer reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_enterer_reference_identifier",
        null=True,
    )
    practitioner = models.ForeignKey(
        "practitioners.Practitioner",
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_enterer_reference_practitioner",
        null=True,
    )
    practitioner_role = models.ForeignKey(
        "practitionerroles.PractitionerRole",
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_enterer_reference_practitioner_role",
        null=True,
    )
    patient = models.ForeignKey(
        "patients.Patient",
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_enterer_reference_patient",
        null=True,
    )
    related_person = models.ForeignKey(
        "relatedpersons.RelatedPerson",
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_enterer_reference_related_person",
        null=True,
    )


class ExplanationOfBenefitProviderReference(BaseReference):
    """Explanation of Benefit provider reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_provider_reference_identifier",
        null=True,
    )
    practitioner = models.ForeignKey(
        "practitioners.Practitioner",
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_provider_reference_practitioner",
        null=True,
    )
    practitioner_role = models.ForeignKey(
        "practitionerroles.PractitionerRole",
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_provider_reference_practitioner_role",
        null=True,
    )
    organization = models.ForeignKey(
        "organizations.Organization",
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_provider_reference_organization",
        null=True,
    )


class ExplanationOfBenefitRelated(TimeStampedModel):
    """Explanation of Benefits related model."""

    claim = models.ForeignKey(
        "claims.ClaimReference",
        on_delete=models.CASCADE,
        related_name="explanation_of_benefits_related_claim",
        null=True,
    )
    relationship = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefits_related_relationship",
        null=True,
    )
    reference = models.ForeignKey(
        Identifier,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefits_related_reference",
        null=True,
    )


class ExplanationOfBenefitPrescriptionReference(BaseReference):
    """Explanation of Benefit prescription reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_prescription_reference_identifier",
        null=True,
    )
    medication_request = models.ForeignKey(
        "medicationrequests.MedicationRequest",
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_prescription_reference_medication_request",
        null=True,
    )
    vision_prescription = models.ForeignKey(
        "visionprescriptions.VisionPrescription",
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_prescription_reference_vision_prescription",
        null=True,
    )


class ExplanationOfBenefitEvent(TimeStampedModel):
    """Explanation of Benefit event model."""

    type = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_event_code",
        null=True,
    )
    when_date_time = models.DateTimeField(null=True)
    when_period = models.ForeignKey(
        Period,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_event_when_period",
        null=True,
    )


class ExplanationOfBenefitPayeePartyReference(BaseReference):
    """Explanation of Benefit payee party reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_payee_party_reference_identifier",
        null=True,
    )
    patient = models.ForeignKey(
        "patients.Patient",
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_payee_party_reference_patient",
        null=True,
    )
    related_person = models.ForeignKey(
        "relatedpersons.RelatedPerson",
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_payee_party_reference_related_person",
        null=True,
    )
    organization = models.ForeignKey(
        "organizations.Organization",
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_payee_party_reference_organization",
        null=True,
    )
    practitioner = models.ForeignKey(
        "practitioners.Practitioner",
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_payee_party_reference_practitioner",
        null=True,
    )
    practitioner_role = models.ForeignKey(
        "practitionerroles.PractitionerRole",
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_payee_party_reference_practitioner_role",
        null=True,
    )


class ExplanationOfBenefitPayee(TimeStampedModel):
    """Explanation of Benefit payee model."""

    type = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_payee_type",
        null=True,
    )
    party = models.ForeignKey(
        ExplanationOfBenefitPayeePartyReference,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_payee_party",
        null=True,
    )


class ExplanationOfBenefitCareTeam(TimeStampedModel):
    """Explanation of Benefit care team model."""

    sequence = models.PositiveIntegerField()
    provider = models.ForeignKey(
        ExplanationOfBenefitProviderReference,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_care_team_provider",
        null=True,
    )
    responsible = models.BooleanField(default=False)
    role = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_care_team_role",
        null=True,
    )
    specialty = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_care_team_qualification",
        null=True,
    )


class ExplanationOfBenefitSupportingInfo(TimeStampedModel):
    """Explanation of Benefit supporting info model."""

    sequence = models.PositiveIntegerField()
    category = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_supporting_info_category",
        null=True,
    )
    code = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_supporting_info_code",
        null=True,
    )
    timing_date = models.DateField(null=True)
    timing_period = models.ForeignKey(
        Period,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_supporting_info_timing_period",
        null=True,
    )
    value_boolean = models.BooleanField(null=True)
    value_string = models.CharField(max_length=255, null=True)
    value_quantity = models.ForeignKey(
        Quantity,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_supporting_info_value_quantity",
        null=True,
    )
    value_attachment = models.ForeignKey(
        Attachment,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_supporting_info_value_attachment",
        null=True,
    )
    value_reference = models.ForeignKey(
        Reference,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_supporting_info_value_reference",
        null=True,
    )
    value_identifier = models.ForeignKey(
        Identifier,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_supporting_info_value_identifier",
        null=True,
    )
    reason = models.ForeignKey(
        Coding,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_supporting_info_reason",
        null=True,
    )


class ExplanationOfBenefitDiagnosis(TimeStampedModel):
    """Explanation of Benefit diagnosis model."""

    sequence = models.PositiveIntegerField()
    diagnosis_codeable_concept = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_diagnosis_diagnosis_codeable_concept",
        null=True,
    )
    diagnosis_reference = models.ForeignKey(
        "conditions.ConditionReference",
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_diagnosis_diagnosis_reference",
        null=True,
    )
    type = models.ManyToManyField(
        CodeableConcept,
        related_name="explanation_of_benefit_diagnosis_type",
        blank=True,
    )
    on_admission = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_diagnosis_on_admission",
        null=True,
    )


class ExplanationOfBenefitProcedure(TimeStampedModel):
    """Explanation of Benefit procedure model."""

    sequence = models.PositiveIntegerField()
    type = models.ManyToManyField(
        CodeableConcept,
        related_name="explanation_of_benefit_procedure_type",
        blank=True,
    )
    date = models.DateTimeField()
    procedure_codeable_concept = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_procedure_procedure_codeable_concept",
        null=True,
    )
    procedure_reference = models.ForeignKey(
        "procedures.ProcedureReference",
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_procedure_procedure_reference",
        null=True,
    )
    udi = models.ManyToManyField(
        "devices.DeviceReference",
        related_name="explanation_of_benefit_procedure_udi",
        blank=True,
    )


class ExplanationOfBenefitInsurance(TimeStampedModel):
    """Explanation of Benefit insurance model."""

    focal = models.BooleanField(default=False)
    coverage = models.ForeignKey(
        "coverages.CoverageReference",
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_insurance_coverage",
        null=True,
    )
    pre_auth_ref = ArrayField(models.CharField(max_length=255), null=True)


class ExplanationOfBenefitAccident(TimeStampedModel):
    """Explanation of Benefit accident model."""

    date = models.DateField(null=True)
    type = models.ForeignKey(
        CodeableConcept,
        related_name="explanation_of_benefit_accident_type",
        on_delete=models.CASCADE,
        null=True,
    )
    location_address = models.ForeignKey(
        Address,
        related_name="explanation_of_benefit_accident_location_address",
        on_delete=models.CASCADE,
        null=True,
    )
    location_reference = models.ForeignKey(
        "locations.LocationReference",
        related_name="explanation_of_benefit_accident_location_location",
        on_delete=models.CASCADE,
        null=True,
    )


class ExplanationOfBenefitItemRequestReference(BaseReference):
    """Explanation of Benefit item request reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_item_request_reference_identifier",
        null=True,
    )
    device_request = models.ForeignKey(
        "devicerequests.DeviceRequest",
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_item_request_reference_device_request",
        null=True,
    )
    medication_request = models.ForeignKey(
        "medicationrequests.MedicationRequest",
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_item_request_reference_medication_request",
        null=True,
    )
    nutrition_order = models.ForeignKey(
        "nutritionorders.NutritionOrder",
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_item_request_reference_nutrition_order",
        null=True,
    )

    service_request = models.ForeignKey(
        "servicerequests.ServiceRequest",
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_item_request_reference_service_request",
        null=True,
    )
    supply_request = models.ForeignKey(
        "supplyrequests.SupplyRequest",
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_item_request_reference_supply_request",
        null=True,
    )
    vision_prescription = models.ForeignKey(
        "visionprescriptions.VisionPrescription",
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_item_request_reference_vision_prescription",
        null=True,
    )


class ExplanationOfBenefitItemBodySite(TimeStampedModel):
    """Explanation of Benefit item body site model."""

    site = models.ManyToManyField(
        "bodystructures.BodyStructureCodeableReference",
        related_name="explanation_of_benefit_item_body_site_site",
        blank=True,
    )
    sub_site = models.ManyToManyField(
        CodeableConcept,
        related_name="explanation_of_benefit_item_body_site_sub_site",
        blank=True,
    )


class ExplanationOfBenefitItemReviewOutcome(TimeStampedModel):
    """Explanation of Benefit item review outcome model."""

    decision = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_item_review_outcome_decision",
        null=True,
    )
    reason = models.ManyToManyField(
        CodeableConcept,
        related_name="explanation_of_benefit_item_review_outcome_reason",
        blank=True,
    )
    pre_auth_ref = models.CharField(max_length=255)
    pre_auth_period = models.ForeignKey(
        Period,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_item_review_outcome_pre_auth_period",
        null=True,
    )


class ExplanationOfBenefitItemAdjudication(TimeStampedModel):
    """Explanation of Benefit item adjudication model."""

    category = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_item_adjudication_category",
        null=True,
    )
    reason = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_item_adjudication_reason",
        null=True,
    )
    amount = models.ForeignKey(
        Money,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_item_adjudication_amount",
        null=True,
    )
    quantity = models.ForeignKey(
        Quantity,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_item_adjudication_quantity",
        null=True,
    )


class ExplanationOfBenefitItemDetailSubDetail(TimeStampedModel):
    """Explanation of Benefit item detail sub detail model."""

    sequence = models.PositiveIntegerField()
    trace_number = models.ManyToManyField(
        Identifier,
        related_name="explanation_of_benefit_item_detail_sub_detail_trace_number",
        blank=True,
    )

    revenue = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_item_detail_sub_detail_revenue",
        null=True,
    )
    category = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_item_detail_sub_detail_category",
        null=True,
    )
    product_or_service = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_item_detail_sub_detail_product_or_service",
        null=True,
    )
    product_or_service_end = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_item_detail_sub_detail_product_or_service_end",
        null=True,
    )
    modifier = models.ManyToManyField(
        CodeableConcept,
        related_name="explanation_of_benefit_item_detail_sub_detail_modifier",
        blank=True,
    )
    program_code = models.ManyToManyField(
        CodeableConcept,
        related_name="explanation_of_benefit_item_detail_sub_detail_program_code",
        blank=True,
    )
    patient_paid = models.ForeignKey(
        Money,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_item_detail_sub_detail_patient_paid",
        null=True,
    )
    quantity = models.ForeignKey(
        SimpleQuantity,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_item_detail_sub_detail_quantity",
        null=True,
    )
    unit_price = models.ForeignKey(
        Money,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_item_detail_sub_detail_unit_price",
        null=True,
    )
    factor = models.DecimalField(max_digits=10, decimal_places=2, null=True)
    tax = models.ForeignKey(
        Money,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_item_detail_sub_detail_tax",
        null=True,
    )
    net = models.ForeignKey(
        Money,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_item_detail_sub_detail_net",
        null=True,
    )
    udi = models.ManyToManyField(
        "devices.DeviceReference",
        related_name="explanation_of_benefit_item_detail_sub_detail_udi",
        blank=True,
    )
    note_number = ArrayField(
        models.PositiveIntegerField(),
        null=True,
    )
    review_outcome = models.ForeignKey(
        ExplanationOfBenefitItemReviewOutcome,
        related_name="explanation_of_benefit_item_detail_sub_detail_review_outcome",
        on_delete=models.CASCADE,
        null=True,
    )
    adjudication = models.ManyToManyField(
        ExplanationOfBenefitItemAdjudication,
        related_name="explanation_of_benefit_item_detail_sub_detail_adjudication",
        blank=True,
    )


class ExplanationOfBenefitItemDetail(TimeStampedModel):
    """Explanation of Benefit item detail model."""

    sequence = models.PositiveIntegerField()
    trace_number = models.ManyToManyField(
        Identifier,
        related_name="explanation_of_benefit_item_detail_trace_number",
        blank=True,
    )
    revenue = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_item_detail_revenue",
        null=True,
    )
    category = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_item_detail_category",
        null=True,
    )
    product_or_service = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_item_detail_product_or_service",
        null=True,
    )
    product_or_service_end = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_item_detail_product_or_service_end",
        null=True,
    )
    modifier = models.ManyToManyField(
        CodeableConcept,
        related_name="explanation_of_benefit_item_detail_modifier",
        blank=True,
    )
    program_code = models.ManyToManyField(
        CodeableConcept,
        related_name="explanation_of_benefit_item_detail_program_code",
        blank=True,
    )
    patient_paid = models.ForeignKey(
        Money,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_item_detail_patient_paid",
        null=True,
    )
    quantity = models.ForeignKey(
        SimpleQuantity,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_item_detail_quantity",
        null=True,
    )
    unit_price = models.ForeignKey(
        Money,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_item_detail_unit_price",
        null=True,
    )
    factor = models.DecimalField(max_digits=10, decimal_places=2, null=True)
    tax = models.ForeignKey(
        Money,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_item_detail_tax",
        null=True,
    )
    net = models.ForeignKey(
        Money,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_item_detail_net",
        null=True,
    )
    udi = models.ManyToManyField(
        "devices.DeviceReference",
        related_name="explanation_of_benefit_item_detail_udi",
        blank=True,
    )
    note_number = ArrayField(
        models.PositiveIntegerField(),
        null=True,
    )
    review_outcome = models.ForeignKey(
        ExplanationOfBenefitItemReviewOutcome,
        related_name="explanation_of_benefit_item_detail_review_outcome",
        on_delete=models.CASCADE,
        null=True,
    )
    adjudication = models.ManyToManyField(
        ExplanationOfBenefitItemAdjudication,
        related_name="explanation_of_benefit_item_detail_adjudication",
        blank=True,
    )
    sub_detail = models.ManyToManyField(
        ExplanationOfBenefitItemDetailSubDetail,
        related_name="explanation_of_benefit_item_detail_sub_detail",
        blank=True,
    )


class ExplanationOfBenefitItem(TimeStampedModel):
    """Explanation of benefit item."""

    sequence = models.PositiveIntegerField()
    care_team_sequence = ArrayField(models.PositiveIntegerField(), null=True)
    diagnosis_sequence = ArrayField(models.PositiveIntegerField(), null=True)
    procedure_sequence = ArrayField(models.PositiveIntegerField(), null=True)
    information_sequence = ArrayField(models.PositiveIntegerField(), null=True)
    trace_number = models.ManyToManyField(
        Identifier, related_name="explanation_of_benefit_item_trace_number", blank=True
    )
    revenue = models.ForeignKey(
        CodeableConcept,
        related_name="explanation_of_benefit_item_revenue",
        on_delete=models.CASCADE,
        null=True,
    )
    category = models.ForeignKey(
        CodeableConcept,
        related_name="explanation_of_benefit_item_category",
        on_delete=models.CASCADE,
        null=True,
    )
    product_or_service = models.ForeignKey(
        CodeableConcept,
        related_name="explanation_of_benefit_item_product_or_service",
        on_delete=models.CASCADE,
        null=True,
    )
    product_or_service_end = models.ForeignKey(
        CodeableConcept,
        related_name="explanation_of_benefit_item_product_or_service_end",
        on_delete=models.CASCADE,
        null=True,
    )
    request = models.ManyToManyField(
        ExplanationOfBenefitItemRequestReference,
        related_name="explanation_of_benefit_item_request",
        blank=True,
    )
    modifier = models.ManyToManyField(
        CodeableConcept, related_name="explanation_of_benefit_item_modifier", blank=True
    )
    program_code = models.ManyToManyField(
        CodeableConcept,
        related_name="explanation_of_benefit_item_program_code",
        blank=True,
    )
    serviced_date = models.DateField(null=True)
    serviced_period = models.ForeignKey(
        Period,
        related_name="explanation_of_benefit_item_serviced_period",
        on_delete=models.CASCADE,
        null=True,
    )
    location_codeable_concept = models.ForeignKey(
        CodeableConcept,
        related_name="explanation_of_benefit_item_location_codeable_concept",
        on_delete=models.CASCADE,
        null=True,
    )
    location_address = models.ForeignKey(
        Address,
        related_name="explanation_of_benefit_item_location_address",
        on_delete=models.CASCADE,
        null=True,
    )
    location_reference = models.ForeignKey(
        "locations.LocationReference",
        related_name="explanation_of_benefit_item_location_reference",
        on_delete=models.CASCADE,
        null=True,
    )
    patient_paid = models.ForeignKey(
        Money,
        related_name="explanation_of_benefit_item_patient_paid",
        on_delete=models.CASCADE,
        null=True,
    )
    quantity = models.ForeignKey(
        SimpleQuantity,
        related_name="explanation_of_benefit_item_quantity",
        on_delete=models.CASCADE,
        null=True,
    )
    unit_price = models.ForeignKey(
        Money,
        related_name="explanation_of_benefit_item_unit_price",
        on_delete=models.CASCADE,
        null=True,
    )
    factor = models.DecimalField(max_digits=10, decimal_places=2, null=True)
    tax = models.ForeignKey(
        Money,
        related_name="explanation_of_benefit_item_tax",
        on_delete=models.CASCADE,
        null=True,
    )
    net = models.ForeignKey(
        Money,
        related_name="explanation_of_benefit_item_net",
        on_delete=models.CASCADE,
        null=True,
    )
    udi = models.ManyToManyField(
        "devices.DeviceReference",
        related_name="explanation_of_benefit_item_udi",
        blank=True,
    )
    body_site = models.ManyToManyField(
        ExplanationOfBenefitItemBodySite,
        related_name="explanation_of_benefit_item_body_site",
        blank=True,
    )
    encounter = models.ManyToManyField(
        "encounters.EncounterReference",
        related_name="explanation_of_benefit_item_encounter",
        blank=True,
    )
    note_number = ArrayField(
        models.PositiveIntegerField(),
        null=True,
    )
    review_outcome = models.ForeignKey(
        ExplanationOfBenefitItemReviewOutcome,
        related_name="explanation_of_benefit_item_review_outcome",
        on_delete=models.CASCADE,
        null=True,
    )
    adjudication = models.ManyToManyField(
        ExplanationOfBenefitItemAdjudication,
        related_name="explanation_of_benefit_item_adjudication",
        blank=True,
    )
    detail = models.ManyToManyField(
        ExplanationOfBenefitItemDetail,
        related_name="explanation_of_benefit_item_detail",
        blank=True,
    )


class ExplanationOfBenefitAddItemDetailSubDetail(TimeStampedModel):
    """Explanation of Benefit add item detail sub detail model."""

    trace_number = models.ManyToManyField(
        Identifier,
        related_name="explanation_of_benefit_add_item_detail_sub_detail_trace_number",
        blank=True,
    )
    revenue = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_add_item_detail_sub_detail_revenue",
        null=True,
    )
    product_or_service = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_add_item_detail_sub_detail_product_or_service",
        null=True,
    )
    product_or_service_end = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_add_item_detail_sub_detail_product_or_service_end",
        null=True,
    )
    modifier = models.ManyToManyField(
        CodeableConcept,
        related_name="explanation_of_benefit_add_item_detail_sub_detail_modifier",
        blank=True,
    )
    patient_paid = models.ForeignKey(
        Money,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_add_item_detail_sub_detail_patient_paid",
        null=True,
    )
    quantity = models.ForeignKey(
        SimpleQuantity,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_add_item_detail_sub_detail_quantity",
        null=True,
    )
    unit_price = models.ForeignKey(
        Money,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_add_item_detail_sub_detail_unit_price",
        null=True,
    )
    factor = models.DecimalField(max_digits=10, decimal_places=2, null=True)
    tax = models.ForeignKey(
        Money,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_add_item_detail_sub_detail_tax",
        null=True,
    )
    net = models.ForeignKey(
        Money,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_add_item_detail_sub_detail_net",
        null=True,
    )
    note_number = ArrayField(
        models.PositiveIntegerField(),
        null=True,
    )
    review_outcome = models.ForeignKey(
        ExplanationOfBenefitItemReviewOutcome,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_add_item_detail_sub_detail_review_outcome",
        null=True,
    )
    adjudication = models.ManyToManyField(
        ExplanationOfBenefitItemAdjudication,
        related_name="explanation_of_benefit_add_item_detail_sub_detail_adjudication",
        blank=True,
    )


class ExplanationOfBenefitAddItemDetail(TimeStampedModel):
    """Explanation of Benefit add item detail model."""

    trace_number = models.ManyToManyField(
        Identifier,
        related_name="explanation_of_benefit_add_item_detail_trace_number",
        blank=True,
    )
    revenue = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_add_item_detail_revenue",
        null=True,
    )
    product_or_service = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_add_item_detail_product_or_service",
        null=True,
    )
    product_or_service_end = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_add_item_detail_product_or_service_end",
        null=True,
    )
    modifier = models.ManyToManyField(
        CodeableConcept,
        related_name="explanation_of_benefit_add_item_detail_modifier",
        blank=True,
    )
    patient_paid = models.ForeignKey(
        Money,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_add_item_detail_patient_paid",
        null=True,
    )
    quantity = models.ForeignKey(
        SimpleQuantity,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_add_item_detail_quantity",
        null=True,
    )
    unit_price = models.ForeignKey(
        Money,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_add_item_detail_unit_price",
        null=True,
    )
    factor = models.DecimalField(max_digits=10, decimal_places=2, null=True)
    tax = models.ForeignKey(
        Money,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_add_item_detail_tax",
        null=True,
    )
    net = models.ForeignKey(
        Money,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_add_item_detail_net",
        null=True,
    )
    note_number = ArrayField(
        models.PositiveIntegerField(),
        null=True,
    )
    review_outcome = models.ForeignKey(
        ExplanationOfBenefitItemReviewOutcome,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_add_item_detail_review_outcome",
        null=True,
    )
    adjudication = models.ManyToManyField(
        ExplanationOfBenefitItemAdjudication,
        related_name="explanation_of_benefit_add_item_detail_adjudication",
        blank=True,
    )
    sub_detail = models.ManyToManyField(
        ExplanationOfBenefitAddItemDetailSubDetail,
        related_name="explanation_of_benefit_add_item_detail_sub_detail",
        blank=True,
    )


class ExplanationOfBenefitAddItem(TimeStampedModel):
    """Explanation of Benefit add item model."""

    item_sequence = ArrayField(models.PositiveIntegerField(), null=True)
    detail_sequence = ArrayField(models.PositiveIntegerField(), null=True)
    sub_detail_sequence = ArrayField(models.PositiveIntegerField(), null=True)
    trace_number = models.ManyToManyField(
        Identifier,
        related_name="explanation_of_benefit_add_item_trace_number",
        blank=True,
    )
    provider = models.ManyToManyField(
        ExplanationOfBenefitProviderReference,
        related_name="explanation_of_benefit_add_item_provider",
        blank=True,
    )
    revenue = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_add_item_revenue",
        null=True,
    )
    product_or_service = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_add_item_product_or_service",
        null=True,
    )
    product_or_service_end = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_add_item_product_or_service_end",
        null=True,
    )
    request = models.ManyToManyField(
        ExplanationOfBenefitItemRequestReference,
        related_name="explanation_of_benefit_add_item_request",
        blank=True,
    )
    modifier = models.ManyToManyField(
        CodeableConcept,
        related_name="explanation_of_benefit_add_item_modifier",
        blank=True,
    )
    program_code = models.ManyToManyField(
        CodeableConcept,
        related_name="explanation_of_benefit_add_item_program_code",
        blank=True,
    )
    serviced_date = models.DateField(null=True)
    serviced_period = models.ForeignKey(
        Period,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_add_item_serviced_period",
        null=True,
    )
    location_codeable_concept = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_add_item_location_codeable_concept",
        null=True,
    )
    location_address = models.ForeignKey(
        Address,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_add_item_location_address",
        null=True,
    )
    location_reference = models.ForeignKey(
        "locations.LocationReference",
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_add_item_location_reference",
        null=True,
    )
    patient_paid = models.ForeignKey(
        Money,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_add_item_patient_paid",
        null=True,
    )
    quantity = models.ForeignKey(
        SimpleQuantity,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_add_item_quantity",
        null=True,
    )
    unit_price = models.ForeignKey(
        Money,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_add_item_unit_price",
        null=True,
    )
    factor = models.DecimalField(max_digits=10, decimal_places=2, null=True)
    tax = models.ForeignKey(
        Money,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_add_item_tax",
        null=True,
    )
    net = models.ForeignKey(
        Money,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_add_item_net",
        null=True,
    )
    body_site = models.ManyToManyField(
        ExplanationOfBenefitItemBodySite,
        related_name="explanation_of_benefit_add_item_body_site",
        blank=True,
    )
    note_number = ArrayField(
        models.PositiveIntegerField(),
        null=True,
    )
    review_outcome = models.ForeignKey(
        ExplanationOfBenefitItemReviewOutcome,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_add_item_review_outcome",
        null=True,
    )
    adjudication = models.ManyToManyField(
        ExplanationOfBenefitItemAdjudication,
        related_name="explanation_of_benefit_add_item_adjudication",
        blank=True,
    )
    detail = models.ManyToManyField(
        ExplanationOfBenefitAddItemDetail,
        related_name="explanation_of_benefit_add_item_detail",
        blank=True,
    )


class ExplanationOfBenefitTotal(TimeStampedModel):
    """Explanation of Benefit total model."""

    category = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_total_category",
        null=True,
    )
    amount = models.ForeignKey(
        Money,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_total_amount",
        null=True,
    )


class ExplanationOfBenefitPayment(TimeStampedModel):
    """Explanation of Benefit payment model."""

    type = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_payment_type",
        null=True,
    )
    adjustment = models.ForeignKey(
        Money,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_payment_adjustment",
        null=True,
    )
    adjustment_reason = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_payment_adjustment_reason",
        null=True,
    )
    date = models.DateField()
    amount = models.ForeignKey(
        Money,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_payment_amount",
        null=True,
    )
    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_payment_identifier",
        null=True,
    )


class ExplanationOfBenefitProcessNote(TimeStampedModel):
    """Explanation of Benefit process note model."""

    number = models.PositiveIntegerField()
    type = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_process_note_type",
        null=True,
    )
    text = models.TextField()
    language = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_process_note_language",
        null=True,
    )


class ExplanationOfBenefitBenefitBalanceFinancial(TimeStampedModel):
    """Explanation of Benefit benefit balance financial model."""

    type = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_benefit_balance_financial_type",
        null=True,
    )
    allowed_unsigned_int = models.PositiveIntegerField(null=True)
    allowed_string = models.CharField(max_length=255, null=True)
    allowed_money = models.ForeignKey(
        Money,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_benefit_balance_financial_allowed_money",
        null=True,
    )
    used_unsigned_int = models.PositiveIntegerField(null=True)
    used_money = models.ForeignKey(
        Money,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_benefit_balance_financial_used_money",
        null=True,
    )


class ExplanationOfBenefitBenefitBalance(TimeStampedModel):
    """Explanation of Benefit benefit balance model."""

    category = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_benefit_balance_category",
        null=True,
    )
    excluded = models.BooleanField(default=False)
    name = models.CharField(max_length=255, null=True)
    description = models.TextField()
    network = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_benefit_balance_network",
        null=True,
    )
    unit = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_benefit_balance_unit",
        null=True,
    )
    term = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefit_benefit_balance_term",
        null=True,
    )
    financial = models.ManyToManyField(
        ExplanationOfBenefitBenefitBalanceFinancial,
        related_name="explanation_of_benefit_benefit_balance_financial",
        blank=True,
    )


class ExplanationOfBenefit(TimeStampedModel):
    """Explanation of Benefits model."""

    identifier = models.ManyToManyField(
        Identifier,
        related_name="explanation_of_benefits_identifier",
        blank=True,
    )
    trace_number = models.ManyToManyField(
        Identifier,
        related_name="explanation_of_benefits_trace_number",
        blank=True,
    )
    status = models.CharField(
        max_length=255,
        choices=choices.ExplanationOfBenefitStatus.choices,
        default=choices.ExplanationOfBenefitStatus.ACTIVE,
    )
    type = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefits_type",
        null=True,
    )
    sub_type = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefits_sub_type",
        null=True,
    )
    use = models.CharField(
        max_length=255,
        choices=choices.ExplanationOfBenefitUse.choices,
        default=choices.ExplanationOfBenefitUse.CLAIM,
    )
    patient = models.ForeignKey(
        "patients.PatientReference",
        on_delete=models.CASCADE,
        related_name="explanation_of_benefits_patient",
        null=True,
    )
    billable_period = models.ForeignKey(
        Period,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefits_billable_period",
        null=True,
    )
    created = models.DateTimeField()
    enterer = models.ForeignKey(
        ExplanationOfBenefitEntererReference,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefits_enterer",
        null=True,
    )
    insurer = models.ForeignKey(
        OrganizationReference,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefits_insurer",
        null=True,
    )
    provider = models.ForeignKey(
        ExplanationOfBenefitProviderReference,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefits_provider",
        null=True,
    )
    priority = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefits_priority",
        null=True,
    )
    funds_reserve_requested = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefits_funds_reserve_requested",
        null=True,
    )
    funds_reserve = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefits_funds_reserve",
        null=True,
    )
    related = models.ManyToManyField(
        ExplanationOfBenefitRelated,
        related_name="explanation_of_benefits_funds_related",
        blank=True,
    )
    prescription = models.ForeignKey(
        ExplanationOfBenefitPrescriptionReference,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefits_prescription",
        null=True,
    )
    original_prescription = models.ForeignKey(
        "medicationrequests.MedicationRequestReference",
        on_delete=models.CASCADE,
        related_name="explanation_of_benefits_original_prescription",
        null=True,
    )
    event = models.ManyToManyField(
        ExplanationOfBenefitEvent,
        related_name="explanation_of_benefits_event",
        blank=True,
    )
    payee = models.ForeignKey(
        ExplanationOfBenefitPayee,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefits_payee",
        null=True,
    )
    referral = models.ForeignKey(
        "servicerequests.ServiceRequestReference",
        on_delete=models.CASCADE,
        related_name="explanation_of_benefits_referral",
        null=True,
    )
    encounter = models.ManyToManyField(
        "encounters.EncounterReference",
        related_name="explanation_of_benefits_encounter",
        blank=True,
    )
    facility = models.ForeignKey(
        "locations.LocationOrganizationReference",
        on_delete=models.CASCADE,
        related_name="explanation_of_benefits_facility",
        null=True,
    )
    claim = models.ForeignKey(
        "claims.ClaimReference",
        on_delete=models.CASCADE,
        related_name="explanation_of_benefits_claim",
        null=True,
    )
    claim_response = models.ForeignKey(
        "claimresponses.ClaimResponseReference",
        on_delete=models.CASCADE,
        related_name="explanation_of_benefits_claim_response",
        null=True,
    )
    outcome = models.CharField(
        max_length=255,
        choices=choices.ExplanationOfBenefitOutcome.choices,
        default=choices.ExplanationOfBenefitOutcome.COMPLETE,
    )
    decision = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefits_decision",
        null=True,
    )
    disposition = models.TextField(null=True)
    pre_auth_ref = ArrayField(models.CharField(max_length=255), null=True)
    pre_auth_period = models.ManyToManyField(
        Period,
        related_name="explanation_of_benefits_pre_auth_period",
        blank=True,
    )
    diagnosis_related_group = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="explanation_of_benefits_diagnosis_related_group",
        null=True,
    )
    care_team = models.ManyToManyField(
        ExplanationOfBenefitCareTeam,
        related_name="explanation_of_benefits_care_team",
        blank=True,
    )
    supporting_info = models.ManyToManyField(
        ExplanationOfBenefitSupportingInfo,
        related_name="explanation_of_benefits_supporting_info",
        blank=True,
    )
    diagnosis = models.ManyToManyField(
        ExplanationOfBenefitDiagnosis,
        related_name="explanation_of_benefits_diagnosis",
        blank=True,
    )
    procedure = models.ManyToManyField(
        ExplanationOfBenefitProcedure,
        related_name="explanation_of_benefits_procedure",
        blank=True,
    )
    precedence = models.PositiveIntegerField(null=True)
    insurance = models.ManyToManyField(
        ExplanationOfBenefitInsurance,
        related_name="explanation_of_benefits_insurance",
        blank=True,
    )
    accident = models.ForeignKey(
        ExplanationOfBenefitAccident,
        related_name="explanation_of_benefits_accident",
        on_delete=models.CASCADE,
        null=True,
    )
    patient_paid = models.ForeignKey(
        Money,
        related_name="explanation_of_benefits_patient_paid",
        on_delete=models.CASCADE,
        null=True,
    )
    item = models.ManyToManyField(
        ExplanationOfBenefitItem,
        related_name="explanation_of_benefits_item",
        blank=True,
    )
    add_item = models.ManyToManyField(
        ExplanationOfBenefitAddItem,
        related_name="explanation_of_benefits_add_item",
        blank=True,
    )
    adjudication = models.ManyToManyField(
        ExplanationOfBenefitItemAdjudication,
        related_name="explanation_of_benefits_adjudication",
        blank=True,
    )
    total = models.ManyToManyField(
        ExplanationOfBenefitTotal,
        related_name="explanation_of_benefits_total",
        blank=True,
    )
    payment = models.ForeignKey(
        ExplanationOfBenefitPayment,
        related_name="explanation_of_benefits_payment",
        on_delete=models.CASCADE,
        null=True,
    )
    form_code = models.ForeignKey(
        CodeableConcept,
        related_name="explanation_of_benefits_form_code",
        on_delete=models.CASCADE,
        null=True,
    )
    form = models.ForeignKey(
        Attachment,
        related_name="explanation_of_benefits_form",
        on_delete=models.CASCADE,
        null=True,
    )
    process_note = models.ManyToManyField(
        ExplanationOfBenefitProcessNote,
        related_name="explanation_of_benefits_process_note",
        blank=True,
    )
    benefit_period = models.ForeignKey(
        Period,
        related_name="explanation_of_benefits_benefit_period",
        on_delete=models.CASCADE,
        null=True,
    )
    benefit_balance = models.ManyToManyField(
        ExplanationOfBenefitBenefitBalance,
        related_name="explanation_of_benefits_benefit_balance",
        blank=True,
    )



"""provenances URLs."""

from django.urls import path

from dfhir.provenances import views

app_name = "provenances"

urlpatterns = [
    path("provenances/", views.ProvenanceListView.as_view()),
    path("provenances/<int:pk>/", views.ProvenanceDetailView.as_view()),
]



"""Flag urls."""

from django.urls import path

from . import views

app_name = "flags"

urlpatterns = [
    path("flags/", views.FlagListView.as_view()),
    path("flags/<int:pk>/", views.FlagDetailView.as_view()),
]



"""molecular sequences views."""

from django.http import Http404
from drf_spectacular.utils import extend_schema
from rest_framework import status
from rest_framework.permissions import AllowAny
from rest_framework.response import Response
from rest_framework.views import APIView

from dfhir.molecularsequences.models import MolecularSequence
from dfhir.molecularsequences.serializers import MolecularSequenceSerializer


class MolecularSequenceListView(APIView):
    """MolecularSequence list view."""

    permission_classes = [AllowAny]

    @extend_schema(responses={200: MolecularSequenceSerializer(many=True)})
    def get(self, request):
        """Get molecular sequences."""
        molecular_sequences = MolecularSequence.objects.all()
        serializer = MolecularSequenceSerializer(molecular_sequences, many=True)
        return Response(serializer.data)

    @extend_schema(
        request=MolecularSequenceSerializer,
        responses={201: MolecularSequenceSerializer},
    )
    def post(self, request):
        """Create a molecular sequence."""
        serializer = MolecularSequenceSerializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data, status=status.HTTP_201_CREATED)


class MolecularSequenceDetailView(APIView):
    """molecular sequence detail view."""

    def get_object(self, pk):
        """Get a molecular sequence object."""
        try:
            return MolecularSequence.objects.get(pk=pk)
        except MolecularSequence.DoesNotExist as err:
            raise Http404 from err

    @extend_schema(responses={200: MolecularSequenceSerializer})
    def get(self, request, pk):
        """Get a molecular sequence."""
        molecular_sequence = self.get_object(pk)
        serializer = MolecularSequenceSerializer(molecular_sequence)
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(
        request=MolecularSequenceSerializer,
        responses={200: MolecularSequenceSerializer},
    )
    def patch(self, request, pk):
        """Update a molecular sequence."""
        molecular_sequence = self.get_object(pk)
        serializer = MolecularSequenceSerializer(
            molecular_sequence, data=request.data, partial=True
        )
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(responses={204: None})
    def delete(self, request, pk):
        """Delete a molecular sequence."""
        molecular_sequence = self.get_object(pk)
        molecular_sequence.delete()
        return Response(status=status.HTTP_204_NO_CONTENT)



"""CoverageEligibilityRequest serializers."""

from drf_writable_nested.serializers import WritableNestedModelSerializer

from dfhir.base.serializers import (
    BaseReferenceModelSerializer,
    BaseWritableNestedModelSerializer,
    CodeableConceptSerializer,
    IdentifierSerializer,
    MoneySerializer,
    OrganizationReferenceSerializer,
    PeriodSerializer,
    SimpleQuantitySerializer,
)
from dfhir.conditions.serializers import ConditionReferenceSerializer
from dfhir.coverages.serializers import CoverageReferenceSerializer
from dfhir.locations.serializers import (
    LocationOrganizationReferenceSerializer,
    LocationReferenceSerializer,
)
from dfhir.patients.serializers import PatientReferenceSerializer
from dfhir.practitioners.serializers import (
    PractitionerPractitionerRoleReferenceSerializer,
)

from .models import (
    CoverageEligibilityRequest,
    CoverageEligibilityRequestEvent,
    CoverageEligibilityRequestInsurance,
    CoverageEligibilityRequestItem,
    CoverageEligibilityRequestItemDetailReference,
    CoverageEligibilityRequestItemDiagnosis,
    CoverageEligibilityRequestProviderReference,
    CoverageEligibilityRequestSupportingInfo,
    CoverageEligibilityRequestSupportingInfoInformationReference,
)


class CoverageEligibilityRequestItemDetailReferenceSerializer(
    BaseReferenceModelSerializer
):
    """CoverageEligibilityRequestItemDetailReference serializer."""

    identifier = IdentifierSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = CoverageEligibilityRequestItemDetailReference
        exclude = ["created_at", "updated_at"]


class CoverageEligibilityRequestProviderReferenceSerializer(
    BaseReferenceModelSerializer
):
    """CoverageEligibilityRequestProviderReference serializer."""

    identifier = IdentifierSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = CoverageEligibilityRequestProviderReference
        exclude = ["created_at", "updated_at"]


class CoverageEligibilityRequestItemDiagnosisSerializer(WritableNestedModelSerializer):
    """CoverageEligibilityRequestItemDiagnosis serializer."""

    diagnosis_codeable_concept = CodeableConceptSerializer(many=False, required=False)
    diagnosis_reference = ConditionReferenceSerializer(required=False)

    class Meta:
        """Meta class."""

        model = CoverageEligibilityRequestItemDiagnosis
        exclude = ["created_at", "updated_at"]


class CoverageEligibilityRequestSupportingInfoInformationReferenceSerializer(
    BaseReferenceModelSerializer
):
    """CoverageEligibilityRequestSupportingInfoInformationReference serializer."""

    identifier = IdentifierSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = CoverageEligibilityRequestSupportingInfoInformationReference
        exclude = ["created_at", "updated_at"]


class CoverageEligibilityRequestItemSerializer(WritableNestedModelSerializer):
    """CoverageEligibilityRequestItem serializer."""

    category = CodeableConceptSerializer(many=False, required=False)
    product_or_service = CodeableConceptSerializer(many=False, required=False)
    modifier = CodeableConceptSerializer(many=True, required=False)
    provider = PractitionerPractitionerRoleReferenceSerializer(
        many=False, required=False
    )
    quantity = SimpleQuantitySerializer(many=False, required=False)
    unit_price = MoneySerializer(many=False, required=False)
    facility = LocationOrganizationReferenceSerializer(many=False, required=False)
    diagnosis = CoverageEligibilityRequestItemDiagnosisSerializer(
        many=True, required=False
    )
    detail = CoverageEligibilityRequestItemDetailReferenceSerializer(
        many=True, required=False
    )

    class Meta:
        """Meta class."""

        model = CoverageEligibilityRequestItem
        exclude = ["created_at", "updated_at"]


class CoverageEligibilityRequestEventSerializer(WritableNestedModelSerializer):
    """CoverageEligibilityRequestEvent serializer."""

    type = CodeableConceptSerializer(many=False, required=False)
    when_period = PeriodSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = CoverageEligibilityRequestEvent
        exclude = ["created_at", "updated_at"]


class CoverageEligibilityRequestInsuranceSerializer(WritableNestedModelSerializer):
    """CoverageEligibilityRequestInsurance serializer."""

    coverage = CoverageReferenceSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = CoverageEligibilityRequestInsurance
        exclude = ["created_at", "updated_at"]


class CoverageEligibilityRequestSupportingInfoSerializer(WritableNestedModelSerializer):
    """CoverageEligibilityRequestSupportingInfo serializer."""

    information = (
        CoverageEligibilityRequestSupportingInfoInformationReferenceSerializer(
            many=False, required=False
        )
    )

    class Meta:
        """Meta class."""

        model = CoverageEligibilityRequestSupportingInfo
        exclude = ["created_at", "updated_at"]


class CoverageEligibilityRequestSerializer(BaseWritableNestedModelSerializer):
    """CoverageEligibilityRequest serializer."""

    identifier = IdentifierSerializer(many=True, required=False)
    priority = CodeableConceptSerializer(many=False, required=False)
    patient = PatientReferenceSerializer(many=False, required=False)
    event = CoverageEligibilityRequestEventSerializer(many=True, required=False)
    serviced_period = PeriodSerializer(many=False, required=False)
    enterer = PractitionerPractitionerRoleReferenceSerializer(
        many=False, required=False
    )
    provider = CoverageEligibilityRequestProviderReferenceSerializer(
        many=False, required=False
    )
    insurer = OrganizationReferenceSerializer(many=False, required=False)
    facility = LocationReferenceSerializer(many=False, required=False)
    supporting_info = CoverageEligibilityRequestSupportingInfoSerializer(
        many=True, required=False
    )
    insurance = CoverageEligibilityRequestInsuranceSerializer(many=True, required=False)
    item = CoverageEligibilityRequestItemSerializer(many=True, required=False)

    class Meta:
        """Meta class."""

        model = CoverageEligibilityRequest
        exclude = ["created_at", "updated_at"]



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0003_initial'),
        ('claims', '0001_initial'),
        ('encounters', '0001_initial'),
        ('locations', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='claim',
            name='encounter',
            field=models.ManyToManyField(blank=True, related_name='claim_encounter', to='encounters.encounterreference'),
        ),
        migrations.AddField(
            model_name='claim',
            name='facility',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='claim_facility', to='locations.locationorganizationreference'),
        ),
        migrations.AddField(
            model_name='claim',
            name='funds_reserve',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='claim_funds_reserve', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='claim',
            name='identifier',
            field=models.ManyToManyField(blank=True, related_name='claim_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='claim',
            name='insurer',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='claim_insurer', to='base.organizationreference'),
        ),
    ]



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0003_initial'),
        ('encounters', '0002_initial'),
        ('locations', '0001_initial'),
        ('medications', '0001_initial'),
        ('organizations', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='ImmunizationBasedOnReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ImmunizationInformationSourceRefrence',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ImmunizationPerformer',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ImmunizationPerformerActorReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ImmunizationProgramEligibility',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ImmunizationProtocolApplied',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('series', models.CharField(max_length=255, null=True)),
                ('dose_number', models.CharField(max_length=255, null=True)),
                ('series_dose', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ImmunizationReaction',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('date', models.DateTimeField(null=True)),
                ('reported', models.BooleanField(default=False)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ImmunizationReasonCodeableReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ImmunizationReasonReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Immunization',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('status', models.CharField(choices=[('completed', 'Completed'), ('entered-in-error', 'Entered in Error'), ('not-done', 'Not Done')], max_length=255, null=True)),
                ('lot_number', models.CharField(max_length=255, null=True)),
                ('expiration_date', models.DateField(null=True)),
                ('occurrence_date_time', models.DateTimeField(null=True)),
                ('occurrence_string', models.CharField(max_length=255, null=True)),
                ('primary_source', models.BooleanField(default=False)),
                ('is_subpotent', models.BooleanField(default=False)),
                ('administered_product', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='immunization_administered_product', to='medications.medicationcodeablereference')),
                ('dose_quantity', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='immunization_dose_quantity', to='base.simplequantity')),
                ('encounter', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='immunization_encounter', to='encounters.encounterreference')),
                ('funding_source', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='immunization_funding_source', to='base.codeableconcept')),
                ('identifier', models.ManyToManyField(blank=True, related_name='immunization_identifier', to='base.identifier')),
                ('location', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='immunization_location', to='locations.locationreference')),
                ('manufacturer', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='immunization_manufacturer', to='organizations.organizationcodeablereference')),
                ('note', models.ManyToManyField(blank=True, related_name='immunization_note', to='base.annotation')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]
